@@ -15,6 +15,7 @@
 #include "utils.h"	/* for cell_name */
 
 /* This many styles are reserved */
+/* FIXME FIXME FIXME : Is this version specific ?? Older biffs may need more */
 #define XF_MAGIC_OFFSET (16 + 4)
 
 /* #define NO_DEBUG_EXCEL */
@@ -466,7 +467,8 @@ biff_font_data_new (ExcelWorkbook *wb, BiffQuery *q)
 			fd->fontname, fd->height / 20, fd->color_idx);
 	}
 #endif
-	fd->style_font = 0;
+	fd->style_font = biff_font_data_get_style_font (fd);
+
         fd->index = g_hash_table_size (wb->font_data);
 	if (fd->index >= 4) /* Wierd: for backwards compatibility */
 		fd->index++;
@@ -606,7 +608,7 @@ biff_name_data_get_name (ExcelSheet *sheet, guint16 idx)
 
 	a = sheet->wb->name_data;
 
-	if (a && 0 < idx && idx < a->len && (bnd = g_ptr_array_index (a, idx-1))) {
+	if (a && 0 < idx && idx <= a->len && (bnd = g_ptr_array_index (a, idx-1))) {
 		if (bnd->type == BNDStore && bnd->v.store.data) {
 			char     *duff = "Some Error";
 			ExprTree *tree;
@@ -640,7 +642,10 @@ biff_name_data_get_name (ExcelSheet *sheet, guint16 idx)
 		else
 			return expr_tree_new_constant (value_new_string (bnd->name));
 	} else
+	{
+		g_warning ("EXCEL : %x (of %x) UNKNOWN name\n", idx-1, a->len);
 		return expr_tree_new_constant (value_new_string("Unknown name"));
+	}
 }
 
 static void
@@ -886,19 +891,23 @@ typedef struct _BiffXFData {
  * NB. index 4 is omitted supposedly for backwards compatiblity
  * Returns the font color if there is one.
  **/
-static StyleColor *
-ms_excel_set_cell_font (ExcelSheet *sheet, Cell *cell, BiffXFData *xf)
+static BiffFontData const *
+ms_excel_get_font (ExcelSheet *sheet, guint16 font_idx)
 {
-	BiffFontData *fd = g_hash_table_lookup (sheet->wb->font_data, &xf->font_idx);
+	BiffFontData const *fd = g_hash_table_lookup (sheet->wb->font_data,
+						      &font_idx);
 
-	if (!fd) {
-		printf ("Unknown font idx %d\n", xf->font_idx);
+	g_return_val_if_fail (fd != NULL, NULL); /* flag the problem */
+	g_return_val_if_fail (fd->index != 4, NULL); /* should not exist */
+	return fd;
+}
+
+static StyleColor *
+ms_excel_set_cell_font (ExcelSheet *sheet, Cell *cell, BiffXFData const *xf)
+{
+	BiffFontData const * fd = ms_excel_get_font (sheet, xf->font_idx);
+	if (fd == NULL)
 		return NULL;
-	}
-	g_assert (fd->index != 4);
-
-	if (!fd->style_font)
-		fd->style_font = biff_font_data_get_style_font (fd);
 
 	if (fd->style_font)
 		cell_set_font_from_style (cell, fd->style_font);
@@ -908,42 +917,48 @@ ms_excel_set_cell_font (ExcelSheet *sheet, Cell *cell, BiffXFData *xf)
 	return ms_excel_palette_get (sheet->wb->palette, fd->color_idx, NULL);
 }
 
+static BiffXFData const *
+ms_excel_get_xf (ExcelSheet *sheet, int const xfidx, Cell *cell)
+{
+	BiffXFData const *xf;
+	GPtrArray const * const p = sheet->wb->XF_cell_records;
+	int const idx = xfidx - XF_MAGIC_OFFSET;
+
+	/* Normal cell formatting */
+	if (xfidx == 0)
+		return NULL;
+
+	/* Default cell formatting */
+	if (xfidx == 15)
+    		return NULL;
+
+	/* FIXME : Put in some verbose debug until we figure this out */
+	if (p == NULL ||  0 > idx || idx >= p->len)
+	{
+	    printf ("%s : MISSING XF.  requested %d out of %d\n",
+		    cell_name(cell->col->pos, cell->row->pos),
+		    idx, p->len);
+	}
+	g_return_val_if_fail (p && 0 <= idx && idx < p->len, NULL);
+	xf = g_ptr_array_index (p, idx);
+
+	g_return_val_if_fail (xf, NULL);
+	g_return_val_if_fail (xf->xftype == eBiffXCell, NULL);
+	return xf;
+}
+
 static void
 ms_excel_set_cell_xf (ExcelSheet *sheet, Cell *cell, guint16 xfidx)
 {
-	BiffXFData *xf;
-	GPtrArray *p;
-	guint16 idx = xfidx - XF_MAGIC_OFFSET;
+	BiffXFData const *xf = ms_excel_get_xf (sheet, xfidx, cell);
 	StyleColor *fore, *back, *basefore;
 	int back_index;
 
-	g_return_if_fail (cell->value);
-
-	if (xfidx == 0) {
-/*		printf ("Normal cell formatting\n"); */
+	/* IF this was an error, the message was already printed. */
+	if (xf == NULL)
 		return;
-	}
-	if (xfidx == 15) {
-/*		printf ("Default cell formatting\n"); */
-    		return;
-	}
 
-	p = sheet->wb->XF_cell_records;
-	if (p && p->len > idx)
-		xf = g_ptr_array_index (p, idx);
-        else {
-#ifndef NO_DEBUG_EXCEL
-		if (ms_excel_read_debug > 0) {
-			printf ("FIXME: No XF record for %d out of %d found :-(\n",
-				xfidx, p?p->len:-666);
-		}
-#endif
-	        return;
-	}
-	if (xf->xftype != eBiffXCell)
-	       printf ("FIXME: Error looking up XF\n");
-
-	g_return_if_fail (xf);
+	g_return_if_fail (cell->value);
 
 	/*
 	 * Well set it up then ! FIXME: hack !
@@ -961,15 +976,9 @@ ms_excel_set_cell_xf (ExcelSheet *sheet, Cell *cell, guint16 xfidx)
 		cell_set_border (cell, xf->border_type, tmp);
 	}
 
-	if (xf->format_idx>0) {
-		if (xf->style_format)
-			cell_set_format_from_style (cell, xf->style_format);
-		else {
-			xf->style_format = biff_format_data_lookup (sheet->wb, xf->format_idx);
-			if (xf->style_format)
-				cell_set_format_from_style (cell, xf->style_format);
-		}
-	}
+	if (xf->style_format)
+		cell_set_format_from_style (cell, xf->style_format);
+
 #ifndef NO_DEBUG_EXCEL
 	if (ms_excel_color_debug > 0) {
 		printf ("%s : Pattern = %d\n",
@@ -1076,7 +1085,8 @@ biff_xf_data_new (ExcelWorkbook *wb, BiffQuery *q, eBiff_version ver)
 
 	xf->font_idx = BIFF_GET_GUINT16 (q->data);
 	xf->format_idx = BIFF_GET_GUINT16 (q->data + 2);
-	xf->style_format = 0;
+	xf->style_format = (xf->format_idx > 0)
+	    ? biff_format_data_lookup (wb, xf->format_idx) : NULL;
 
 	data = BIFF_GET_GUINT16 (q->data + 4);
 	xf->locked = (data & 0x0001) ? eBiffLLocked : eBiffLUnlocked;
@@ -1276,7 +1286,7 @@ biff_xf_data_new (ExcelWorkbook *wb, BiffQuery *q, eBiff_version ver)
 		/*printf ("Inserting into Cell XF hash with : %d\n", wb->XF_cell_records->len); */
 		g_ptr_array_add (wb->XF_cell_records, xf);
 	} else {
-		/*printf ("Inserting into style XF hash with : %d\n", wb->XF_style_records->len); */
+		/* printf ("Inserting into style XF hash with : %d\n", wb->XF_style_records->len); */
 		g_ptr_array_add (wb->XF_style_records, xf);
 	}
 #ifndef NO_DEBUG_EXCEL
@@ -1365,8 +1375,15 @@ ms_excel_formula_shared (BiffQuery *q, ExcelSheet *sheet, Cell *cell)
 							 !is_array, data_len,
 							 NULL);
 		BiffSharedFormula *sf = g_new (BiffSharedFormula, 1);
-		sf->key.col = array_col_first;
-		sf->key.row = array_row_first;
+
+		/*
+		 * WARNING : Do NOT use the upper left corner as the hashkey.
+		 *     For some bizzare reason XL appears to sometimes not
+		 *     flag the formula as shared until later.
+		 *  Use the location of the cell we are reading as the key.
+		 */
+		sf->key.col = cell->col->pos; /* array_col_first; */
+		sf->key.row = cell->row->pos; /* array_row_first; */
 		sf->is_array = is_array;
 		if (data_len > 0) {
 			sf->data = g_new (guint8, data_len);
@@ -1388,19 +1405,14 @@ ms_excel_formula_shared (BiffQuery *q, ExcelSheet *sheet, Cell *cell)
 
 		g_return_val_if_fail (expr != NULL, FALSE);
 
-		if (is_array) {
+		if (is_array)
 			cell_set_array_formula (sheet->gnum_sheet,
 						array_row_first,
 						array_col_first,
 						array_row_last,
 						array_col_last, expr);
-		} else {
-			cell = sheet_cell_fetch (sheet->gnum_sheet,
-						 array_col_first,
-						 array_row_first);
-			if (cell)
-				cell_set_formula_tree_simple (cell, expr);
-		}
+		else
+		    cell_set_formula_tree_simple (cell, expr);
 
 		expr_tree_unref (expr);
 		sheet->blank = FALSE;
@@ -1435,7 +1447,7 @@ ms_excel_read_formula (BiffQuery *q, ExcelSheet *sheet)
 
 	/*
 	 * Get the current value so that we can format, do this BEFORE handling
-	 * shared/array formulas or strings incase we need to go to the next
+	 * shared/array formulas or strings in case we need to go to the next
 	 * record
 	 */
 	if (BIFF_GET_GUINT16 (q->data+12) != 0xffff) {
@@ -1557,6 +1569,11 @@ ms_excel_sheet_shared_formula (ExcelSheet *sheet, int const col, int const row)
 	BiffSharedFormulaKey k;
 	k.col = col;
 	k.row = row;
+
+#ifndef NO_DEBUG_EXCEL
+	if (ms_excel_read_debug > 5)
+		printf ("FIND SHARED : %s%d\n", col_name(col), row+1);
+#endif
 	return g_hash_table_lookup (sheet->shared_formulae, &k);
 }
 
@@ -2018,20 +2035,22 @@ ms_excel_read_cell (BiffQuery *q, ExcelSheet *sheet)
 					   EX_GETROW (q), v);
 		break;
 	}
-	case BIFF_COLINFO: /* FIXME: See: S59D67.HTM */
+	case BIFF_COLINFO: /* See: S59D67.HTM */
 	{
-		int firstcol, lastcol, lp;
-		guint16 cols_xf, options, width;
-		int hidden, collapsed, outlining;
-		firstcol = BIFF_GET_GUINT16(q->data);
-		lastcol  = BIFF_GET_GUINT16(q->data+2);
-		width    = BIFF_GET_GUINT16(q->data+4);
-		cols_xf  = BIFF_GET_GUINT16(q->data+6);
-		options  = BIFF_GET_GUINT16(q->data+8);
-
-		hidden    = (options & 0x0001) != 0;
-		collapsed = (options & 0x1000) != 0;
-		outlining = (options & 0x0700) >> 8;
+		int lp;
+		int divisor;
+		BiffXFData const *xf;
+		BiffFontData const *fd;
+		guint16 const firstcol = BIFF_GET_GUINT16(q->data);
+		guint16 const lastcol  = BIFF_GET_GUINT16(q->data+2);
+		guint16       width    = BIFF_GET_GUINT16(q->data+4);
+		guint16 const cols_xf  = BIFF_GET_GUINT16(q->data+6);
+		guint16 const options  = BIFF_GET_GUINT16(q->data+8);
+		gboolean const hidden = (options & 0x0001) ? TRUE : FALSE;
+#if 0
+		gboolean const collapsed = (options & 0x1000) ? TRUE : FALSE;
+		int const outline_level = (options >> 8) & 0x7;
+#endif
 
 #ifndef NO_DEBUG_EXCEL
 		if (ms_excel_read_debug > 1) {
@@ -2042,12 +2061,29 @@ ms_excel_read_cell (BiffQuery *q, ExcelSheet *sheet)
 				firstcol, lastcol, width/256.0);
 		}
 #endif
+		/*
+		 * FIXME FIXME FIXME 
+		 * 1) As a default 36 seems seems to match the sheet I
+		 *    calibrated against.
+		 * 2) the docs say charwidth not height.  Is this correct ?
+		 */
+		if ((xf = ms_excel_get_xf (sheet, cols_xf, NULL)) != NULL &&
+		    (fd = ms_excel_get_font (sheet, xf->font_idx))) {
+			divisor = fd->height / 10;
+		} else
+			divisor = 36.;
+
 		if (width>>8 == 0) {
-			printf ("FIXME: Hidden columns need implementing\n");
-			width=40.0;
+			if (hidden)
+				printf ("FIXME: Hidden column unimplemented\n");
+			else
+				printf ("FIXME: 0 sized column ???\n");
+			width = 62 * divisor;
 		}
+
 		for (lp=firstcol;lp<=lastcol;lp++)
-			sheet_col_set_width (sheet->gnum_sheet, lp, width/25);
+			sheet_col_set_width (sheet->gnum_sheet, lp,
+					     width/divisor);
 		break;
 	}
 	case BIFF_RK: /* See: S59DDA.HTM */
@@ -2418,8 +2454,10 @@ biff_get_externsheet_name(ExcelWorkbook *wb, guint16 idx, gboolean get_first)
 	index = get_first ? bed->first_tab : bed->last_tab;
 
 	bsd = g_hash_table_lookup (wb->boundsheet_data_by_index, &index);
-	if (!bsd)
+	if (!bsd || !bsd->sheet) {
 		printf ("Duff sheet index %d\n", index);
+		return NULL;
+	}
 	return bsd->sheet->gnum_sheet;
 }
 
@@ -2623,6 +2661,8 @@ ms_excel_read_workbook (MsOle *file)
 				}
 			} else if (ver->type == eBiffTChart)
 				ms_excel_chart (q, wb, ver);
+			else if (ver->type == eBiffTVBModule)
+				printf ("VB Module.\n");
 			else
 				printf ("Unknown BOF (%x)\n",ver->type);
 		}
@@ -2649,22 +2689,22 @@ ms_excel_read_workbook (MsOle *file)
 				biff_font_data_new (wb, q);
 			}
 			break;
-		case BIFF_PRECISION:	/*
-					 * FIXME:
-					 */
-#ifndef NO_DEBUG_EXCEL
-			if (ms_excel_read_debug > 0) {
-				printf ("Precision : \n");
-				dump (q->data, q->length);
-			}
+		case BIFF_PRECISION:
+			{
+#if 0
+				/* FIXME : implement in gnumeric */
+				/* state of 'Precision as Displayed' option */
+				guint16 const data = BIFF_GET_GUINT16(q->data);
+				gboolean const prec_as_displayed = (data == 0);
 #endif
+			}
 			break;
-		case BIFF_XF_OLD:	/*
-					 * FIXME: see S59E1E.HTM
-					 */
+
+		case BIFF_XF_OLD: /* see S59E1E.HTM */
 		case BIFF_XF:
 			biff_xf_data_new (wb, q, ver->version);
 			break;
+
 		case BIFF_SST: /* see S59DE7.HTM */
 			{
 				guint32 length, k, tot_len;
@@ -2879,9 +2919,10 @@ ms_excel_read_workbook (MsOle *file)
 		case BIFF_SCL :
 			break;
 
-		case BIFF_MS_O_DRAWING: /* FIXME: See: ms-escher.c and S59DA4.HTM */
-			if (gnumeric_debugging>0)
-				ms_escher_hack_get_drawing (q);
+		case BIFF_MS_O_DRAWING:
+		case BIFF_MS_O_DRAWING_GROUP:
+		case BIFF_MS_O_DRAWING_SELECTION:
+			ms_escher_hack_get_drawing (q);
 			break;
 
 		default:
