@@ -1,3 +1,6 @@
+#ifndef GNUMERIC_MS_OBJ_H
+#define GNUMERIC_MS_OBJ_H
+
 /**
  * ms-obj.h: MS Excel Graphic Object support for Gnumeric
  *
@@ -5,9 +8,8 @@
  *    Michael Meeks (michael@imaginator.com)
  **/
 
-#ifndef GNUMERIC_MS_OBJ_H
-#define GNUMERIC_MS_OBJ_H
+#include "ms-excel-read.h"
 
 void ms_obj_read_obj (BiffQuery *q, ExcelWorkbook * wb);
 
-#endif
+#endif /* GNUMERIC_MS_OBJ_H */
