@@ -12,7 +12,7 @@
 #include "../../src/plugin.h"
 
 static Value *
-func_plusone (FunctionDefinition * fndef, Value *argv [], char **error_string)
+func_plusone (FunctionEvalInfo *ei, Value *argv [])
 {
 	Value *v = g_new (Value, 1);
 	
@@ -22,11 +22,6 @@ func_plusone (FunctionDefinition * fndef, Value *argv [], char **error_string)
 	return v;
 }
 
-static FunctionDefinition plugin_functions [] = {
-	{ "plusone",     "f",    "number",    NULL, NULL, func_plusone },
-	{ NULL, NULL },
-};
-
 static int
 can_unload (PluginData *pd)
 {
@@ -51,7 +46,10 @@ cleanup_plugin (PluginData *pd)
 int
 init_plugin (PluginData *pd)
 {
-	install_symbols (plugin_functions, "Sample Plugin");
+	FunctionCategory *cat = function_get_category (_("Sample Plugin"));
+
+	function_add_args (cat, "plusone", "f", "number", NULL, func_plusone);
+
 	pd->can_unload = can_unload;
 	pd->cleanup_plugin = cleanup_plugin;
 	pd->title = g_strdup ("PlusOne Plugin");
