@@ -229,6 +229,22 @@ free_data_set(data_set_t *data)
 	g_slist_free(data->array);
 }
 
+static void
+prepare_output(Workbook *wb, data_analysis_output_t *dao, char *name)
+{
+	if (dao->type == NewSheetOutput) {
+	        dao->sheet = sheet_new(wb, name);
+		dao->start_col = dao->start_row = 0;
+		workbook_attach_sheet(wb, dao->sheet);
+	} else if (dao->type == NewWorkbookOutput) {
+		wb = workbook_new ();
+		dao->sheet = sheet_new(wb, name);
+		dao->start_col = dao->start_row = 0;
+		workbook_attach_sheet(wb, dao->sheet);
+		gtk_widget_show (wb->toplevel);
+	}
+}
+
 
 /************* Correlation Tool *******************************************
  *
@@ -278,7 +294,7 @@ correl(data_set_t *set_one, data_set_t *set_two, int *error_flag)
  * otherwise by rows.
  */
 int
-correlation_tool (Workbook *wb, Sheet *current_sheet, 
+correlation_tool (Workbook *wb, Sheet *sheet, 
 		  Range *input_range, int columns_flag,
 		  data_analysis_output_t *dao)
 {
@@ -288,53 +304,70 @@ correlation_tool (Workbook *wb, Sheet *current_sheet,
 	int        vars, cols, rows, col, row, i;
 	int        error;
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
-
 	cols = input_range->end_col - input_range->start_col + 1;
 	rows = input_range->end_row - input_range->start_row + 1;
 
+	prepare_output(wb, dao, "Correlations");
+
 	set_cell (dao, 0, 0, "");
 
 	if (columns_flag) {
 	        vars = cols;
-		for (col=0; col<vars; col++) {
-		        sprintf(buf, "Column %d", col+1);
-			cell = set_cell (dao, 0, col+1, buf);
-		}
-		for (row=0; row<vars; row++) {
-		        sprintf(buf, "Column %d", row+1);
-			cell = set_cell (dao, 1+row, 0, buf);
-		}
-		data_sets = g_new(data_set_t, vars);
+		if (dao->labels_flag) {
+		        rows--;
+			for (col=0; col<vars; col++) {
+			        char *s;
+			        cell = sheet_cell_get
+				  (sheet, input_range->start_col+col, 
+				   input_range->start_row);
+				if (cell != NULL && cell->value != NULL) {
+				        s = value_get_as_string(cell->value);
+				        set_cell (dao, 0, col+1, s);
+				        set_cell (dao, col+1, 0, s);
+				} else
+				        return 1;
+			}
+			input_range->start_row++;
+		} else
+		        for (col=0; col<vars; col++) {
+			        sprintf(buf, "Column %d", col+1);
+				set_cell (dao, 0, col+1, buf);
+				set_cell (dao, col+1, 0, buf);
+			}
 
+		data_sets = g_new(data_set_t, vars);
 		for (i=0; i<vars; i++)
-		        get_data_groupped_by_columns(current_sheet,
+		        get_data_groupped_by_columns(sheet,
 						     input_range, i, 
 						     &data_sets[i]);
 	} else {
 	        vars = rows;
-		for (col=0; col<vars; col++) {
-		        sprintf(buf, "Row %d", col+1);
-			cell = set_cell (dao, 0, col+1, buf);
-		}
-		for (row=0; row<vars; row++) {
-		        sprintf(buf, "Row %d", row+1);
-			cell = set_cell (dao, 1+row, 0, buf);
-		}
+		if (dao->labels_flag) {
+		        cols--;
+			for (col=0; col<vars; col++) {
+			        char *s;
+			        cell = sheet_cell_get
+				  (sheet, input_range->start_col,
+				   input_range->start_row+col);
+				if (cell != NULL && cell->value != NULL) {
+				        s = value_get_as_string(cell->value);
+				        set_cell (dao, 0, col+1, s);
+				        set_cell (dao, col+1, 0, s);
+				} else
+				        return 1;
+			}
+			input_range->start_col++;
+		} else 
+		        for (col=0; col<vars; col++) {
+			        sprintf(buf, "Row %d", col+1);
+				set_cell (dao, 0, col+1, buf);
+				set_cell (dao, col+1, 0, buf);
+			}
+ 
 		data_sets = g_new(data_set_t, vars);
 
 		for (i=0; i<vars; i++)
-		        get_data_groupped_by_rows(current_sheet,
+		        get_data_groupped_by_rows(sheet,
 						  input_range, i, 
 						  &data_sets[i]);
 	}
@@ -349,7 +382,7 @@ correlation_tool (Workbook *wb, Sheet *current_sheet,
 							  &data_sets[row],
 							  &error));
 				if (error)
-				        set_cell (dao, col+1, row+1, "--");
+				        set_cell (dao, col+1, row+1, "#N/A");
 				else
 				        set_cell (dao, col+1, row+1, buf);
 			}
@@ -408,7 +441,7 @@ covar(data_set_t *set_one, data_set_t *set_two, int *error_flag)
  * otherwise by rows.
  */
 int
-covariance_tool (Workbook *wb, Sheet *current_sheet, 
+covariance_tool (Workbook *wb, Sheet *sheet, 
 		 Range *input_range, int columns_flag,
 		 data_analysis_output_t *dao)
 {
@@ -418,17 +451,7 @@ covariance_tool (Workbook *wb, Sheet *current_sheet,
 	int        vars, cols, rows, col, row, i;
 	int        error;
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "Covariances");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "Covariances");
 
 	cols = input_range->end_col - input_range->start_col + 1;
 	rows = input_range->end_row - input_range->start_row + 1;
@@ -437,49 +460,75 @@ covariance_tool (Workbook *wb, Sheet *current_sheet,
 
 	if (columns_flag) {
 	        vars = cols;
-		for (col=0; col<vars; col++) {
-		        sprintf(buf, "Column %d", col+1);
-			cell = set_cell (dao, 0, col+1, buf);
-		}
-		for (row=0; row<vars; row++) {
-		        sprintf(buf, "Column %d", row+1);
-			cell = set_cell (dao, 1+row, 0, buf);
-		}
-		data_sets = g_new(data_set_t, vars);
+		if (dao->labels_flag) {
+		        rows--;
+			for (col=0; col<vars; col++) {
+			        char *s;
+			        cell = sheet_cell_get
+				  (sheet, input_range->start_col+col, 
+				   input_range->start_row);
+				if (cell != NULL && cell->value != NULL) {
+				        s = value_get_as_string(cell->value);
+				        set_cell (dao, 0, col+1, s);
+				        set_cell (dao, col+1, 0, s);
+				} else
+				        return 1;
+			}
+			input_range->start_row++;
+		} else
+		        for (col=0; col<vars; col++) {
+			        sprintf(buf, "Column %d", col+1);
+				set_cell (dao, 0, col+1, buf);
+				set_cell (dao, col+1, 0, buf);
+			}
 
+		data_sets = g_new(data_set_t, vars);
 		for (i=0; i<vars; i++)
-		        get_data_groupped_by_columns(current_sheet,
+		        get_data_groupped_by_columns(sheet,
 						     input_range, i, 
 						     &data_sets[i]);
 	} else {
 	        vars = rows;
-		for (col=0; col<vars; col++) {
-		        sprintf(buf, "Row %d", col+1);
-			cell = set_cell (dao, 0, col+1, buf);
-		}
-		for (row=0; row<vars; row++) {
-		        sprintf(buf, "Row %d", row+1);
-			cell = set_cell (dao, 1+row, 0, buf);
-		}
+		if (dao->labels_flag) {
+		        cols--;
+			for (col=0; col<vars; col++) {
+			        char *s;
+			        cell = sheet_cell_get
+				  (sheet, input_range->start_col,
+				   input_range->start_row+col);
+				if (cell != NULL && cell->value != NULL) {
+				        s = value_get_as_string(cell->value);
+				        set_cell (dao, 0, col+1, s);
+				        set_cell (dao, col+1, 0, s);
+				} else
+				        return 1;
+			}
+			input_range->start_col++;
+		} else 
+		        for (col=0; col<vars; col++) {
+			        sprintf(buf, "Row %d", col+1);
+				set_cell (dao, 0, col+1, buf);
+				set_cell (dao, col+1, 0, buf);
+			}
+ 
 		data_sets = g_new(data_set_t, vars);
 
 		for (i=0; i<vars; i++)
-		        get_data_groupped_by_rows(current_sheet,
+		        get_data_groupped_by_rows(sheet,
 						  input_range, i, 
 						  &data_sets[i]);
 	}
 
 	for (row=0; row<vars; row++) {
 		  for (col=0; col<vars; col++) {
-		        if (row == col) {
-			        set_cell (dao, col+1, row+1, "1");
+		        if (row < col)
 				break;
-			} else {
+			else {
 			        sprintf(buf, "%f", covar(&data_sets[col],
 							 &data_sets[row],
 							  &error));
 				if (error)
-				        set_cell (dao, col+1, row+1, "--");
+				        set_cell (dao, col+1, row+1, "#N/A");
 				else
 				        set_cell (dao, col+1, row+1, buf);
 			}
@@ -573,17 +622,7 @@ summary_statistics(Workbook *wb, data_set_t *data_set, int vars,
 	float_t x;
 	int     col, error;
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "Summary Statistics");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "Summary Statistics");
 
         set_cell (dao, 0, 0, "");
 	for (col=0; col<vars; col++) {
@@ -696,17 +735,7 @@ confidence_level(Workbook *wb, data_set_t *data_set, int vars, float_t c_level,
         char    buf[256];
         int col;
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "Confidence Level");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "Confidence Level");
 
 	for (col=0; col<vars; col++) {
 	        float_t stdev = sqrt((data_set[col].sqrsum - 
@@ -732,17 +761,7 @@ kth_largest(Workbook *wb, data_set_t *data_set, int vars, int k,
         char    buf[256];
         int     col;
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "kth Largest");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "Kth Largest");
 
 	for (col=0; col<vars; col++) {
 	        sprintf(buf, "Column %d", col+1);
@@ -768,17 +787,7 @@ kth_smallest(Workbook *wb, data_set_t *data_set, int vars, int k,
         char    buf[256];
         int     col;
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "kth Smallest");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "Kth Smallest");
 
 	for (col=0; col<vars; col++) {
 	        sprintf(buf, "Column %d", col+1);
@@ -826,12 +835,21 @@ descriptive_stat_tool (Workbook *wb, Sheet *current_sheet,
 						  &data_sets[i]);
 	}
 
-        if (ds->summary_statistics)
+        if (ds->summary_statistics) {
                 summary_statistics(wb, data_sets, vars, dao);
-        if (ds->confidence_level)
+		if (dao->type == RangeOutput)
+		        dao->start_row += 15;
+	}
+        if (ds->confidence_level) {
                 confidence_level(wb, data_sets, vars, ds->c_level, dao);
-        if (ds->kth_largest)
+		if (dao->type == RangeOutput)
+		        dao->start_row += 4;
+	}
+        if (ds->kth_largest) {
                 kth_largest(wb, data_sets, vars, ds->k_largest, dao);
+		if (dao->type == RangeOutput)
+		        dao->start_row += 4;
+	}
         if (ds->kth_smallest)
                 kth_smallest(wb, data_sets, vars, ds->k_smallest, dao);
 
@@ -867,17 +885,7 @@ int sampling_tool (Workbook *wb, Sheet *sheet, Range *input_range,
 	char       buf[256];
 	float_t    x;
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "Sample");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "Sample");
 
 	get_data(sheet, input_range, &data_set);
 
@@ -988,24 +996,36 @@ int ztest_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	float_t    mean1, mean2, var1, var2, z, p;
 	char       buf[256];
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "z-Test");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "z-Test");
 
 	get_data(sheet, input_range1, &set_one);
 	get_data(sheet, input_range2, &set_two);
 
         set_cell (dao, 0, 0, "");
-        set_cell (dao, 1, 0, "Variable 1");
-        set_cell (dao, 2, 0, "Variable 2");
+
+	if (dao->labels_flag) {
+	        char *s;
+		Cell *cell;
+
+		cell = sheet_cell_get(sheet, input_range1->start_col, 
+				      input_range1->start_row);
+		if (cell != NULL && cell->value != NULL) {
+		        s = value_get_as_string(cell->value);
+			set_cell (dao, 1, 0, s);
+		}
+		cell = sheet_cell_get(sheet, input_range2->start_col, 
+				      input_range2->start_row);
+		if (cell != NULL && cell->value != NULL) {
+		        s = value_get_as_string(cell->value);
+			set_cell (dao, 2, 0, s);
+		}
+
+		input_range1->start_row++;
+		input_range2->start_row++;
+	} else {
+	        set_cell (dao, 1, 0, "Variable 1");
+		set_cell (dao, 2, 0, "Variable 2");
+	}
 
         set_cell (dao, 0, 1, "Mean");
         set_cell (dao, 0, 2, "Known Variance");
@@ -1056,7 +1076,7 @@ int ztest_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	set_cell(dao, 1, 6, buf);
 
 	/* z Critical one-tail */
-	sprintf(buf, "%f", 0.0);    /* TODO */
+	sprintf(buf, "%f", qnorm(alpha, 0, 1));
 	set_cell(dao, 1, 7, buf);
 
 	/* P(Z<=z) two-tail */
@@ -1064,7 +1084,7 @@ int ztest_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	set_cell(dao, 1, 8, buf);
 
 	/* z Critical two-tail */
-	sprintf(buf, "%f", 0.0);    /* TODO */
+	sprintf(buf, "%f", qnorm(1.0-(1.0-alpha)/2, 0, 1));
 	set_cell(dao, 1, 9, buf);
 
 	free_data_set (&set_one);
@@ -1109,21 +1129,33 @@ ttest_paired_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	        return 1;
 	}
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "t-Test");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "t-Test");
 
         set_cell (dao, 0, 0, "");
-        set_cell (dao, 1, 0, "Variable 1");
-        set_cell (dao, 2, 0, "Variable 2");
+
+	if (dao->labels_flag) {
+	        char *s;
+		Cell *cell;
+
+		cell = sheet_cell_get(sheet, input_range1->start_col, 
+				      input_range1->start_row);
+		if (cell != NULL && cell->value != NULL) {
+		        s = value_get_as_string(cell->value);
+			set_cell (dao, 1, 0, s);
+		}
+		cell = sheet_cell_get(sheet, input_range2->start_col, 
+				      input_range2->start_row);
+		if (cell != NULL && cell->value != NULL) {
+		        s = value_get_as_string(cell->value);
+			set_cell (dao, 2, 0, s);
+		}
+
+		input_range1->start_row++;
+		input_range2->start_row++;
+	} else {
+	        set_cell (dao, 1, 0, "Variable 1");
+		set_cell (dao, 2, 0, "Variable 2");
+	}
 
         set_cell (dao, 0, 1, "Mean");
         set_cell (dao, 0, 2, "Variance");
@@ -1212,7 +1244,7 @@ ttest_paired_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	set_cell(dao, 1, 8, buf);
 
 	/* t Critical one-tail */
-	sprintf(buf, "%f", 0.0);     /* TODO */
+	sprintf(buf, "%f", qt(alpha, df));
 	set_cell(dao, 1, 9, buf);
 
 	/* P(T<=t) two-tail */
@@ -1220,7 +1252,7 @@ ttest_paired_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	set_cell(dao, 1, 10, buf);
 
 	/* t Critical two-tail */
-	sprintf(buf, "%f", 0.0);     /* TODO */
+	sprintf(buf, "%f", qt(1.0-(1.0-alpha)/2, df));
 	set_cell(dao, 1, 11, buf);
 
         free_data_set(&set_one);
@@ -1241,24 +1273,36 @@ ttest_eq_var_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	float_t    mean1, mean2, var, var1, var2, t, p, df;
 	char       buf[256];
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "t-Test");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "t-Test");
 
 	get_data(sheet, input_range1, &set_one);
 	get_data(sheet, input_range2, &set_two);
 
         set_cell (dao, 0, 0, "");
-        set_cell (dao, 1, 0, "Variable 1");
-        set_cell (dao, 2, 0, "Variable 2");
+
+	if (dao->labels_flag) {
+	        char *s;
+		Cell *cell;
+
+		cell = sheet_cell_get(sheet, input_range1->start_col, 
+				      input_range1->start_row);
+		if (cell != NULL && cell->value != NULL) {
+		        s = value_get_as_string(cell->value);
+			set_cell (dao, 1, 0, s);
+		}
+		cell = sheet_cell_get(sheet, input_range2->start_col, 
+				      input_range2->start_row);
+		if (cell != NULL && cell->value != NULL) {
+		        s = value_get_as_string(cell->value);
+			set_cell (dao, 2, 0, s);
+		}
+
+		input_range1->start_row++;
+		input_range2->start_row++;
+	} else {
+	        set_cell (dao, 1, 0, "Variable 1");
+		set_cell (dao, 2, 0, "Variable 2");
+	}
 
         set_cell (dao, 0, 1, "Mean");
         set_cell (dao, 0, 2, "Variance");
@@ -1326,7 +1370,7 @@ ttest_eq_var_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	set_cell(dao, 1, 8, buf);
 
 	/* t Critical one-tail */
-	sprintf(buf, "%f", 0.0);     /* TODO */
+	sprintf(buf, "%f", qt(alpha, df));
 	set_cell(dao, 1, 9, buf);
 
 	/* P(T<=t) two-tail */
@@ -1334,7 +1378,7 @@ ttest_eq_var_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	set_cell(dao, 1, 10, buf);
 
 	/* t Critical two-tail */
-	sprintf(buf, "%f", 0.0);     /* TODO */
+	sprintf(buf, "%f", qt(1.0-(1.0-alpha)/2, df));
 	set_cell(dao, 1, 11, buf);
 
         free_data_set(&set_one);
@@ -1355,24 +1399,36 @@ ttest_neq_var_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	float_t    mean1, mean2, var1, var2, t, p, df, c;
 	char       buf[256];
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "t-Test");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "t-Test");
 
 	get_data(sheet, input_range1, &set_one);
 	get_data(sheet, input_range2, &set_two);
 
         set_cell (dao, 0, 0, "");
-        set_cell (dao, 1, 0, "Variable 1");
-        set_cell (dao, 2, 0, "Variable 2");
+
+	if (dao->labels_flag) {
+	        char *s;
+		Cell *cell;
+
+		cell = sheet_cell_get(sheet, input_range1->start_col, 
+				      input_range1->start_row);
+		if (cell != NULL && cell->value != NULL) {
+		        s = value_get_as_string(cell->value);
+			set_cell (dao, 1, 0, s);
+		}
+		cell = sheet_cell_get(sheet, input_range2->start_col, 
+				      input_range2->start_row);
+		if (cell != NULL && cell->value != NULL) {
+		        s = value_get_as_string(cell->value);
+			set_cell (dao, 2, 0, s);
+		}
+
+		input_range1->start_row++;
+		input_range2->start_row++;
+	} else {
+	        set_cell (dao, 1, 0, "Variable 1");
+		set_cell (dao, 2, 0, "Variable 2");
+	}
 
         set_cell (dao, 0, 1, "Mean");
         set_cell (dao, 0, 2, "Variance");
@@ -1433,7 +1489,7 @@ ttest_neq_var_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	set_cell(dao, 1, 7, buf);
 
 	/* t Critical one-tail */
-	sprintf(buf, "%f", 0.0);     /* TODO */
+	sprintf(buf, "%f", qt(alpha, df));
 	set_cell(dao, 1, 8, buf);
 
 	/* P(T<=t) two-tail */
@@ -1441,7 +1497,7 @@ ttest_neq_var_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	set_cell(dao, 1, 9, buf);
 
 	/* t Critical two-tail */
-	sprintf(buf, "%f", 0.0);     /* TODO */
+	sprintf(buf, "%f", qt(1.0-(1.0-alpha)/2, df));
 	set_cell(dao, 1, 10, buf);
 
         free_data_set(&set_one);
@@ -1470,25 +1526,37 @@ ftest_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	float_t    mean1, mean2, var1, var2, f, p, df1, df2, c;
 	char       buf[256];
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "F-Test");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
+	prepare_output(wb, dao, "F-Test");
+
+        set_cell (dao, 0, 0, "");
+
+	if (dao->labels_flag) {
+	        char *s;
+		Cell *cell;
+
+		cell = sheet_cell_get(sheet, input_range1->start_col, 
+				      input_range1->start_row);
+		if (cell != NULL && cell->value != NULL) {
+		        s = value_get_as_string(cell->value);
+			set_cell (dao, 1, 0, s);
+		}
+		cell = sheet_cell_get(sheet, input_range2->start_col, 
+				      input_range2->start_row);
+		if (cell != NULL && cell->value != NULL) {
+		        s = value_get_as_string(cell->value);
+			set_cell (dao, 2, 0, s);
+		}
+
+		input_range1->start_row++;
+		input_range2->start_row++;
+	} else {
+	        set_cell (dao, 1, 0, "Variable 1");
+		set_cell (dao, 2, 0, "Variable 2");
 	}
 
 	get_data(sheet, input_range1, &set_one);
 	get_data(sheet, input_range2, &set_two);
 
-        set_cell (dao, 0, 0, "");
-        set_cell (dao, 1, 0, "Variable 1");
-        set_cell (dao, 2, 0, "Variable 2");
-
         set_cell (dao, 0, 1, "Mean");
         set_cell (dao, 0, 2, "Variance");
         set_cell (dao, 0, 3, "Observations");
@@ -1542,8 +1610,8 @@ ftest_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 	sprintf(buf, "%f", p);
 	set_cell(dao, 1, 6, buf);
 
-	/* t Critical one-tail */
-	sprintf(buf, "%f", 0.0);     /* TODO */
+	/* F Critical one-tail */
+	sprintf(buf, "%f", qf(alpha, df1, df2));
 	set_cell(dao, 1, 7, buf);
 
         free_data_set(&set_one);
@@ -1572,17 +1640,7 @@ int random_tool (Workbook *wb, Sheet *sheet, int vars, int count,
 	Value      **values, *v;
 	Cell       *cell;
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "Random Numbers");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "Random");
 
 	switch (distribution) {
 	case DiscreteDistribution:
@@ -1668,6 +1726,16 @@ int random_tool (Workbook *wb, Sheet *sheet, int vars, int count,
 			}
 		}
 	        break;
+	case BernoulliDistribution:
+	        for (i=0; i<vars; i++) {
+		        for (n=0; n<count; n++) {
+			        sprintf(buf, "%d", 
+					(random_01() <= param->bernoulli.p) ?
+					1 : 0);
+				set_cell(dao, i, n, buf);
+			}
+		}
+	        break;
 	case UniformDistribution:
 	        range = param->uniform.upper_limit-param->uniform.lower_limit;
 	        for (i=0; i<vars; i++) {
@@ -1715,18 +1783,8 @@ int regression_tool (Workbook *wb, Sheet *sheet, Range *input_range1,
 		return 1;
 	}
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "Regression");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
-	  
+	prepare_output(wb, dao, "Regression");
+
         set_cell (dao, 0, 0, "SUMMARY OUTPUT");
 
 	set_cell (dao, 0, 2, "Regression Statistics");
@@ -1833,17 +1891,7 @@ int average_tool (Workbook *wb, Sheet *sheet, Range *range, int interval,
 	if ((cols != 1 && rows != 1) || interval < 1)
 	        return 1;
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "Moving Averages");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "Moving Averages");
 
 	prev = g_new(float_t, interval);
 
@@ -1889,26 +1937,33 @@ int average_tool (Workbook *wb, Sheet *sheet, Range *range, int interval,
  *
  **/
 
+typedef struct {
+        int     rank;
+        int     same_rank_count;
+        int     point;
+        float_t x;
+} rank_t;
+
+static gint
+rank_compare (const rank_t *a, const rank_t *b)
+{
+        if (a->x < b->x)
+                return 1;
+        else if (a->x == b->x)
+                return 0;
+        else
+                return -1;
+}
 
 int ranking_tool (Workbook *wb, Sheet *sheet, Range *input_range,
 		  int columns_flag, data_analysis_output_t *dao)
 {
         data_set_t *data_sets;
-	GSList     *current;
+	GSList     *current, *inner;
 	char       buf[256];
 	int        vars, cols, rows, col, i, n;
 
-	if (dao->type == NewSheetOutput) {
-	        dao->sheet = sheet_new(wb, "Ranks and Percentiles");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-	} else if (dao->type == NewWorkbookOutput) {
-		wb = workbook_new ();
-		dao->sheet = sheet_new(wb, "Correlations");
-		dao->start_col = dao->start_row = 0;
-		workbook_attach_sheet(wb, dao->sheet);
-		gtk_widget_show (wb->toplevel);
-	}
+	prepare_output(wb, dao, "Ranks");
 
 	cols = input_range->end_col - input_range->start_col + 1;
 	rows = input_range->end_row - input_range->start_row + 1;
@@ -1917,13 +1972,27 @@ int ranking_tool (Workbook *wb, Sheet *sheet, Range *input_range,
 	        vars = cols;
 		for (col=0; col<vars; col++) {
 		        set_cell (dao, col*4, 0, "Point");
-		        sprintf(buf, "Column %d", col+1);
-			set_cell (dao, col*4+1, 0, buf);
+			if (dao->labels_flag) {
+			        char *s;
+			        Cell *cell = sheet_cell_get
+				  (sheet, input_range->start_col+col, 
+				   input_range->start_row);
+				if (cell != NULL && cell->value != NULL) {
+				        s = value_get_as_string(cell->value);
+				        set_cell (dao, col*4+1, 0, s);
+				}
+			} else {
+			        sprintf(buf, "Column %d", col+1);
+				set_cell (dao, col*4+1, 0, buf);
+			}
 			set_cell (dao, col*4+2, 0, "Rank");
 			set_cell (dao, col*4+3, 0, "Percent");
 		}
 		data_sets = g_new(data_set_t, vars);
 
+		if (dao->labels_flag)
+		        input_range->start_row++;
+
 		for (i=0; i<vars; i++)
 		        get_data_groupped_by_columns(sheet,
 						     input_range, i, 
@@ -1932,39 +2001,83 @@ int ranking_tool (Workbook *wb, Sheet *sheet, Range *input_range,
 	        vars = rows;
 		for (col=0; col<vars; col++) {
 		        set_cell (dao, col*4, 0, "Point");
-		        sprintf(buf, "Row %d", col+1);
-			set_cell (dao, col*4+1, 0, buf);
+
+			if (dao->labels_flag) {
+			        char *s;
+			        Cell *cell = sheet_cell_get
+				  (sheet, input_range->start_col, 
+				   input_range->start_row+col);
+				if (cell != NULL && cell->value != NULL) {
+				        s = value_get_as_string(cell->value);
+				        set_cell (dao, col*4+1, 0, s);
+				}
+			} else {
+		                sprintf(buf, "Row %d", col+1);
+				set_cell (dao, col*4+1, 0, buf);
+			}
 			set_cell (dao, col*4+2, 0, "Rank");
 			set_cell (dao, col*4+3, 0, "Percent");
 		}
 		data_sets = g_new(data_set_t, vars);
 
+		if (dao->labels_flag)
+		        input_range->start_col++;
+
 		for (i=0; i<vars; i++)
 		        get_data_groupped_by_rows(sheet,
 						  input_range, i, 
 						  &data_sets[i]);
 	}
 
-	for (i=0; i<vars; i++)
-	        data_sets[i].array = g_slist_sort(data_sets[i].array, 
-			     (GCompareFunc) float_compare_desc);
-	       
 	for (i=0; i<vars; i++) {
+	        rank_t *rank;
 	        n = 0;
 	        current = data_sets[i].array;
+		rank = g_new(rank_t, data_sets[i].n);
+
 		while (current != NULL) {
 		        float_t x = *((float_t *) current->data);
 
-			sprintf(buf, "%f", x);
-			set_cell (dao, i*4+1, n+1, buf);
-			sprintf(buf, "%d", n+1);
-			set_cell (dao, i*4+2, n+1, buf);
-			sprintf(buf, "%.2f%%", 
-				100.0-(100.0 * n / (data_sets[i].n-1)));
-			set_cell (dao, i*4+3, n+1, buf);
-			++n;
+			rank[n].point = n+1;
+			rank[n].x = x;
+			rank[n].rank = 1;
+			rank[n].same_rank_count = -1;
+
+			inner = data_sets[i].array;
+			while (inner != NULL) {
+			        float_t y = *((float_t *) inner->data);
+				if (y > x)
+				        rank[n].rank++;
+				else if (y == x)
+				        rank[n].same_rank_count++;
+				inner = inner->next;
+			}
+			n++;
 			current = current->next;
 		}
+		qsort ((rank_t *) rank, data_sets[i].n,
+		       sizeof (rank_t), (void *) &rank_compare);
+
+		for (n=0; n<data_sets[i].n; n++) {
+			/* Point number */
+			sprintf(buf, "%d", rank[n].point);
+			set_cell (dao, i*4+0, n+1, buf);
+
+			/* Value */
+			sprintf(buf, "%f", rank[n].x);
+			set_cell (dao, i*4+1, n+1, buf);
+
+			/* Rank */
+			sprintf(buf, "%d", rank[n].rank);
+			set_cell (dao, i*4+2, n+1, buf);
+
+			/* Percent */
+			sprintf(buf, "%.2f%%", 
+				100.0-(100.0 * (rank[n].rank-1)/
+				       (data_sets[i].n-1)));
+			set_cell (dao, i*4+3, n+1, buf);
+		}
+		g_free(rank);
 	}
 
 	for (i=0; i<vars; i++)
