@@ -39,7 +39,6 @@ corba_sheet (Sheet *sheet, CORBA_Environment *ev)
 static GNOME_Gnumeric_Sheet
 Workbook_sheet_new (PortableServer_Servant servant, const CORBA_char * name, CORBA_Environment * ev)
 {
-	GNOME_Gnumeric_Sheet ggs;
         Workbook *workbook = workbook_from_servant (servant);
 	Sheet *sheet;
 
@@ -81,7 +80,7 @@ Workbook_save_to (PortableServer_Servant servant, const CORBA_char * filename, C
 {
         Workbook *workbook = workbook_from_servant (servant);
 
-	gnumericWriteXmlWorkbook (workbook, filename);
+	gnumeric_xml_write_workbook (workbook, filename);
 }
 
 static GNOME_Gnumeric_Sheet
@@ -163,8 +162,8 @@ Workbook_corba_class_init ()
 
 	gnome_gnumeric_workbook_vepv.GNOME_Gnumeric_Workbook_epv =
 		&gnome_gnumeric_workbook_epv;
-	gnome_gnumeric_workbook_vepv.GNOME_obj_epv =
-		&gnome_obj_epv;
+	gnome_gnumeric_workbook_vepv.GNOME_Unknown_epv =
+		&gnome_object_epv;
 
 	gnome_gnumeric_workbook_epv.sheet_new = Workbook_sheet_new;
 	gnome_gnumeric_workbook_epv.sheet_lookup = Workbook_sheet_lookup;
