@@ -36,6 +36,7 @@ static char *help_address = {
 	   "\n"
 	   "If @row_num or @col_num is less than one, ADDRESS returns #NUM! "
 	   "error. "
+	   "\n"
 	   "@SEEALSO=")
 };
 
@@ -70,7 +71,7 @@ gnumeric_address (FunctionEvalInfo *ei, Value **args)
 	        text[0] = '\0';
 	} else {
 	        gchar *p = args[4]->v.str->str;
-		int   n=0, space=0;
+		int   space = 0;
 
 		text = g_new(gchar, strlen(p) + 3);
 		while (*p)
@@ -286,7 +287,7 @@ gnumeric_vlookup (FunctionEvalInfo *ei, Value **args)
 		int compare;
 		const Value *v;
 
-		v = value_area_get_at_x_y (&ei->pos, args[1], 0, lp);
+		v = value_area_fetch_x_y (&ei->pos, args[1], 0, lp);
 
 		g_return_val_if_fail (v != NULL, NULL);
 
@@ -295,7 +296,7 @@ gnumeric_vlookup (FunctionEvalInfo *ei, Value **args)
 		if (compare == 1){
 			const Value *v;
 
-			v = value_area_get_at_x_y (&ei->pos, args [1], col_idx-1, lp);
+			v = value_area_fetch_x_y (&ei->pos, args [1], col_idx-1, lp);
 			g_return_val_if_fail (v != NULL, NULL);
 
 			return value_duplicate (v);
@@ -308,7 +309,7 @@ gnumeric_vlookup (FunctionEvalInfo *ei, Value **args)
 	if (approx && next_largest){
 	        const Value *v;
 
-		v = value_area_get_at_x_y (&ei->pos, args [1], col_idx-1,
+		v = value_area_fetch_x_y (&ei->pos, args [1], col_idx-1,
 					   next_largest_row);
 		g_return_val_if_fail (v != NULL, NULL);
 		return value_duplicate (v);
@@ -366,7 +367,7 @@ gnumeric_hlookup (FunctionEvalInfo *ei, Value **args)
 		int compare;
 		const Value *v;
 
-		v = value_area_get_at_x_y (&ei->pos, args[1],lp, 0);
+		v = value_area_fetch_x_y (&ei->pos, args[1],lp, 0);
 
 		g_return_val_if_fail (v != NULL, NULL);
 
@@ -375,7 +376,7 @@ gnumeric_hlookup (FunctionEvalInfo *ei, Value **args)
 		if (compare == 1){
 			const Value *v;
 
-			v = value_area_get_at_x_y (&ei->pos, args [1], lp, row_idx-1);
+			v = value_area_fetch_x_y (&ei->pos, args [1], lp, row_idx-1);
 			g_return_val_if_fail (v != NULL, NULL);
 
 			return value_duplicate (v);
@@ -389,7 +390,7 @@ gnumeric_hlookup (FunctionEvalInfo *ei, Value **args)
 	if (approx && next_largest){
 		const Value *v;
 
-		v = value_area_get_at_x_y (&ei->pos, args [1],
+		v = value_area_fetch_x_y (&ei->pos, args [1],
 					   next_largest_col, row_idx-1);
 		g_return_val_if_fail (v != NULL, NULL);
 
@@ -467,10 +468,10 @@ gnumeric_lookup (FunctionEvalInfo *ei, Value **args)
 
 		touched = 0;
 		for (lpx=0,lpy=0;lpx<maxx && lpy<maxy;) {
-			const Value *v = value_area_get_at_x_y (&ei->pos, src, lpx, lpy);
+			const Value *v = value_area_fetch_x_y (&ei->pos, src, lpx, lpy);
 			compare = lookup_similar (v, args[0], next_largest, 1);
 			if (compare == 1)
-				return value_duplicate (value_duplicate (value_area_get_at_x_y (&ei->pos, dest, next_largest_x+x_offset,
+				return value_duplicate (value_duplicate (value_area_fetch_x_y (&ei->pos, dest, next_largest_x+x_offset,
 													  next_largest_y+y_offset)));
 			if (compare < 0) {
 				next_largest = v;
@@ -488,7 +489,7 @@ gnumeric_lookup (FunctionEvalInfo *ei, Value **args)
 		if (!next_largest)
 			return function_error (ei, gnumeric_err_NA);
 
-		return value_duplicate (value_area_get_at_x_y (&ei->pos, dest,
+		return value_duplicate (value_area_fetch_x_y (&ei->pos, dest,
 							       next_largest_x+x_offset,
 							       next_largest_y+y_offset));
 	}
