@@ -819,9 +819,13 @@ gnumeric_dollar (FunctionEvalInfo *ei, Value **argv)
 	Value *v, *ag [3];
 	guint len, neg;
 	gchar *s;
+	static int barfed = 0;
 
-	g_warning ("GNUMERIC_DOLLAR is broken, it should use the "
-		   "format_value routine");
+	if (!barfed) {
+		g_warning ("GNUMERIC_DOLLAR is broken, it should use the "
+			   "format_value routine");
+		barfed = 1;
+	}
 	ag[0] = argv [0];
 	ag[1] = argv [1];
 	ag[2] = NULL;
@@ -1077,7 +1081,7 @@ match_again:
 void
 string_functions_init ()
 {
-	FunctionCategory *cat = function_get_category (_("Date / Time"));
+	FunctionCategory *cat = function_get_category (_("String"));
 
 	function_add_args  (cat, "char",       "f",    "number",
 			    &help_char,       gnumeric_char);
