@@ -23,10 +23,16 @@ int range_var_pop (const float_t *xs, int n, float_t *res);
 int range_var_est (const float_t *xs, int n, float_t *res);
 int range_stddev_pop (const float_t *xs, int n, float_t *res);
 int range_stddev_est (const float_t *xs, int n, float_t *res);
-int range_skew (const float_t *xs, int n, float_t *res);
+int range_skew_pop (const float_t *xs, int n, float_t *res);
+int range_skew_est (const float_t *xs, int n, float_t *res);
+int range_kurtosis_m3_pop (const float_t *xs, int n, float_t *res);
+int range_kurtosis_m3_est (const float_t *xs, int n, float_t *res);
 
 /* ------------------------------------------------------------------------- */
 
+double bessel_i (double x, double alpha, double expo);
+double bessel_k (double x, double alpha, double expo);
+
 /* "d": density.  */
 /* "p": distribution function.  */
 /* "q": inverse distribution function.  */
@@ -61,6 +67,23 @@ double qf (double x, double n1, double n2);
 double pchisq (double x, double df);
 double qchisq (double p, double df);
 
+/* The Weibull distribution.  */
+double dweibull (double x, double shape, double scale);
+double pweibull (double x, double shape, double scale);
+
+/* The Poisson distribution.  */
+double dpois (double x, double lambda);
+double ppois (double x, double lambda);
+
+/* The exponential distribution.  */
+double dexp (double x, double scale);
+double pexp (double x, double scale);
+
+/* Binomial distribution.  */
+double dbinom (double x, double n, double p);
+double pbinom (double x, double n, double p);
+double qbinom (double x, double n, double p);
+
 /* ------------------------------------------------------------------------- */
 
 #endif
